//
// Created by huli on 2020/7/20.
//

#ifndef NNXX_TESTS_DISPATCH_MANAGER_H
#define NNXX_TESTS_DISPATCH_MANAGER_H


#include "../error_code/error_code.h"
#include "../tool/singleton.h"
#include "../tool/thread_condition.h"
#include "../tool/TaskQueue/TQFactory.h"
#include "../tool/TaskQueue/BaseTask.h"

#include "../dispatch/carrier.h"
#include "../dispatch/catcher.h"
#include "../dispatch/passageway.h"
#include "../dispatch/dispatch_process.h"

#include <vector>
#include <glog/logging.h>



//lacate测量结果结构体, 整车的信息,
typedef struct Locate_information
{
	float locate_x;				//整车的中心点x值, 四轮的中心
	float locate_y;				//整车的中心点y值, 四轮的中心
	float locate_angle;			//整车的旋转角, 四轮的旋转角
	float locate_length;		//整车的长度, 用于规避碰撞
	float locate_width;			//整车的宽度, 用于规避碰撞
	float locate_height;		//整车的高度, 用于规避碰撞
	float locate_wheel_base;	//整车的轮距, 前后轮的距离, 用于机器人或agv的抓车
	float locate_wheel_width;	//整车的轮距, 左右轮的距离, 用于机器人或agv的抓车
	bool locate_correct;		//整车的校准标记位
	//注:理论上, 车宽和左右轮距应该是一样的, 但是实际上车宽比左右轮距略大,
}Locate_information;

//调度管理模块
class Dispatch_manager:public Singleton<Dispatch_manager>
{
// 子类必须把父类设定为友元函数，这样父类才能使用子类的私有构造函数。
   friend class Singleton<Dispatch_manager>;

public:
	//调度管理 的状态
	enum Dispatch_manager_status
	{
		E_DISPATCH_MANAGER_UNKNOW               = 0,    //未知
		E_DISPATCH_MANAGER_READY                = 1,    //准备，待机
		E_DISPATCH_MANAGER_STORE                = 2,    //正在存车
		E_DISPATCH_MANAGER_PICKUP               = 3,    //正在取车

		E_DISPATCH_MANAGER_FAULT               = 10,    //故障
	};

	//调度方向, 停车取车
	enum Dispatch_motion_direction
	{
		E_STORE_CAR             =0,         //停车, 出入口 -> 停车位
		E_PICKUP_CAR            =1,         //取车, 停车位 -> 出入口
	};
private:
 // 父类的构造函数必须保护，子类的构造函数必须私有。
   Dispatch_manager();
public:
    //必须关闭拷贝构造和赋值构造，只能通过 get_instance 函数来进行操作唯一的实例。
    Dispatch_manager(const Dispatch_manager& other) = delete;
    Dispatch_manager& operator =(const Dispatch_manager& other) = delete;
    ~Dispatch_manager();
public://API functions
	//调度管理 初始化
	Error_manager dispatch_manager_init(int dispatch_id);
	//调度管理 反初始化
	Error_manager dispatch_manager_uninit();

	//对外的接口函数，负责接受并处理任务单，
	Error_manager execute_task(Dispatch_manager::Dispatch_motion_direction dispatch_motion_direction);

	//检查状态
	Error_manager check_status();


	//在流程的map 里面释放指定的流程
	Error_manager release_dispatch_process(std::string command_key);

public://get or set member variable
	Dispatch_manager_status get_dispatch_manager_status();
	int get_dispatch_id();
	void set_dispatch_id(int dispatch_id);
protected://member variable

	Dispatch_manager_status						m_dispatch_manager_status;			//调度管理 的状态
	int 										m_dispatch_id;						//调度模块的id, (楚天项目就是单元号, 0~2)

	//硬件资源
	int 										m_catcher_size;					//抓车器的数量, 默认1个
	std::map<int, std::shared_ptr<Catcher>>		m_catcher_map;					//抓车器的对象实例，内存由本类管理
	int 										m_carrier_size;					//搬运器的数量, 默认3个
	std::map<int, std::shared_ptr<Carrier>>		m_carrier_map;					//搬运器的对象实例，内存由本类管理
	int 										m_passageway_size;				//通道口的数量, 默认7个
	std::map<int, std::shared_ptr<Passageway>>	m_passageway_map;				//通道口的对象实例，内存由本类管理


	//流程控制
	std::mutex                              	m_lock;        						//线程池的锁, 增删流程时要加锁.
	tq::IQueue*                         		m_process_thread_queue;         	//指令流程线程池, 管理线程内存, 负责启动和回收线程内存资源,不负责控制流程.
	std::thread*                        		m_dispatch_manager_thread;        	//调度总管理的线程, 总控全局, 控制每个流程的先后顺序, 并合理的分配资源.
	Thread_condition				    		m_dispatch_manager_condition;		//调度总管理的条件变量, 总控全局, 控制每个流程的先后顺序, 并合理的分配资源.

	//流程管理
	std::map<std::string, Dispatch_process*>	m_key_to_dispatch_process_store_map;	//存车流程的map
	std::map<std::string, Dispatch_process*>	m_key_to_dispatch_process_pickup_map;	//取车流程的map



private:

};


#endif //NNXX_TESTS_DISPATCH_MANAGER_H
